'use strict';

const ApiProvider = require('./provider');
const Controller = require('../controller');

/**
 * Microkit api controller
 */
class ApiController extends Controller {
  get kind() {
    return "api";
  }

  get schema() {
    return Joi.object({
      services: Joi.array().items(Joi.string()).default([])
    });
  }

  get services() {
    return _.mapValues(this.options.services, service => {
      return this.microkit.getProvider(service);
    });
  }

  /**
   * Calls an api
   * @param {string} service service to call
   * @param {string} method method to call
   * @param {object} parameters parameters to pass
   * @param {object} options additional options to pass
   */
  call(service, method, parameters, options) {
    service = _.get(this.services, service);

    Joi.assert(service, Joi.object().type(ApiProvider), 'service not ApiProvider');

    service._initialize();

    return service.ready().then(() => call(method, parameters, options));
  }
}

module.exports = ApiController;
