'use strict';

const Joi = require('joi');
const _ = require('lodash');

const microkit = require('../../');

class ExeceptionHandlerController extends microkit.Controller.mixin(
  microkit.LoggerMixin, microkit.ErrorMixin
) {
  initialize() {
    _.forEach(this.options.events, event => {
      this.installHandler(event, this.options.restartOnException);
    });
  }

  get schema() {
    return Joi.object({
      events: Joi.array().items(Joi.string()).default([]),
      restartOnException: Joi.boolean().default(true)
    });
  }

  /**
   * Installs handler for exception event
   */
  installHandler(event, shouldRestart) {
    process.once(event, err => {
      this.logger.error(event, err);

      return this.error.capture(err, {}, {wait: true}).finally(() => {
        if (shouldRestart) {
          return this.exit().then(() => {
            process.exit(1);
          });
        }
      });
    });
  }
}

module.exports = ExeceptionHandlerController;
