'use strict';

const Joi = require('joi');
const Promise = require('bluebird');

const Controller = require('../controller');

/**
 * Microkit metrics controller
 */
class MetricsController extends Controller {
  get kind() {
    return "metrics";
  }

  /**
   * Sends metric
   * @param {string} name name of the metric
   * @param {integer} value value to send
   * @param {object} tags tag metric with
   * @param {object.unit} options metric unit (ms,km,eur)
   * @param {object.increment} options whether metric should be handled as increment to the previous value
   * @param {object.decrement} options whether metric should be handler as decrement to previous value
   */
  send(name, value, tags, options) {
    return Promise.try(() => {
      options = options || {};

      Joi.assert(name, Joi.string().required(), 'Metric name must be a string');
      Joi.assert(value, Joi.number().required(), 'Metric value must be integer');
      Joi.assert(tags, Joi.object().pattern(/.*/, Joi.string()).optional(), 'Metric tag value must be string');

      // Coerce value to number
      value = Joi.number().validate(value).value;

      return this.provider.send(name, value, tags, options);
    });
  }
}

module.exports = MetricsController;
