'use strict';

const Controller = require('../controller');

/**
 * Controller for queue
 */
class QueueController extends Controller {
  get kind() {
    return "queue";
  }

  get schema() {
    return Joi.object({
      context: Joi.object().default({}).description('context to pass with messages'),
      mergeToBody: Joi.string().default(false).description('whether to merge header to body')
    });
  }

  publish() {
    let key;
    let message;
    let options;

    if (_.has(arguments[0], 'key') && _.has(arguments[0], 'message')) {
      key = arguments[0].key;
      message = arguments[0].message || {};
      options = arguments[0].options || arguments[1];
    } else {
      key = arguments[0];
      message = arguments[1] || {};
      options = arguments[2];
    }

    if (!key) {
      throw new Error('Event key not provided');
    }

    if (_.isObject(key)) {
      _.extend(key, this.options.context);
    }

    if (_.isObject(key) && this.options.mergeToBody) {
      _.extend(message, key);
    }

    _.extend(message, {
      _time: new Date().toISOString()

      // TODO: add message signatures
    });

    return this.provider.publish(key, message, options).catch(err => {
      // if messages cannot be publish report errors
      this.error.capture(err);
      this.error.capture(new Error('cannot publish message'), {key: key, message: message});

      throw err;
    });
  }

  /**
   * Subscribes for queue messages
   *
   * @param {object|string} selector message selector
   * @param {function} handler message handler
   * @param {object} options options to pass to subscriber
   */
  subscribe(selector, handler, options) {
    const _handler = event => {
      return Promise.try(() => {
        return handler.length === 2 ?
          handler(event.data, event) :
          handler(event);
      }).catch(err => {
        this.error.capture(err, {selector: selector});
        throw err;
      });
    };

    return this.provider.subscribe(selector, _handler, options);
  }
}

module.exports = QueueController;
