'use strict';

const Provider = require('../provider');

const errors = require('../errors');

/**
 * Callback called for every queue message
 *
 * @callback QueueProvider~callback
 * @param {object} message Queue message
 * @param {object} key Queue key
 */

/**
 * Microkit queue provider abstract class
 */
class QueueProvider extends Provider {
  /**
   * Publishes message to queue
   *
   * @param {object} key key to publish message to
   * @param {object} message message to publish
   */
  publish(key, message, options) {
    throw new errors.NotImplementedError('publish_not_implemented');
  }

  /**
   * Subcsribes to queue
   * @param {string} key key to subscribe to
   * @param {queueCallback} handler queue handler
   * @param {type} name description
   */
  subscribe(key, handler, options) {
    throw new errors.NotImplementedError('subscribe_not_implemented');
  }
}

module.exports = QueueProvider;
